from openapi_core.deserializing.parameters.factories import (
    ParameterDeserializersFactory,
)

__all__ = ["parameter_deserializers_factory"]

parameter_deserializers_factory = ParameterDeserializersFactory()
